@@ -49,6 +49,7 @@ enum aie2_msg_opcode {
 	MSG_OP_GET_APP_HEALTH			= 0x114,
 	MSG_OP_ADD_HOST_BUFFER			= 0x115,
 	MSG_OP_CONFIG_FW_LOG			= 0x116,
+	MSG_OP_GET_DEV_REVISION			= 0x117,
 	MSG_OP_GET_COREDUMP			= 0x119,
 	MSG_OP_CALIBRATE_TIME			= 0x11C,
 	MSG_OP_MAX_DRV_OPCODE,
@@ -767,4 +768,26 @@ struct calibrate_time_resp {
 	enum aie2_msg_status	status;
 } __packed;
 
+enum aie2_dev_revision {
+	AIE2_DEV_REVISION_STXA = 1,
+	AIE2_DEV_REVISION_STXB,
+	AIE2_DEV_REVISION_KRK1,
+	AIE2_DEV_REVISION_KRK2,
+	AIE2_DEV_REVISION_HALO,
+	AIE2_DEV_REVISION_GPT1,
+	AIE2_DEV_REVISION_GPT2,
+	AIE2_DEV_REVISION_GPT3,
+	AIE2_DEV_REVISION_UNKN,
+};
+
+struct get_dev_revision_req {
+	u32			place_holder;
+} __packed;
+
+struct get_dev_revision_resp {
+	enum aie2_msg_status	status;
+	enum aie2_dev_revision	rev;
+	u32			raw_fuse_data;
+} __packed;
+
 #endif /* _AIE2_MSG_PRIV_H_ */
