@@ -24,18 +24,9 @@ private:
   parse_categories(const std::vector<std::string>& categories_list,
                    const xrt_core::device* device) const;
 
-  std::map<std::string, uint32_t> 
-  get_category_map(const xrt_core::device* device) const;
-
-  std::vector<std::string>
-  mask_to_category_names(uint32_t mask, const xrt_core::device* device) const;
-
   void
   handle_list_categories(const xrt_core::device* device) const;
 
-  void
-  handle_status(const xrt_core::device* device) const;
-
   void
   handle_config(const xrt_core::device* device) const;
 
@@ -45,6 +36,5 @@ private:
   bool m_disable;
   bool m_help;
   bool m_list_categories;
-  bool m_status;
   std::vector<std::string> m_categories;
 };
